# -*- coding: utf-8 -*-
"""
    tests.exceptions
    ~~~~~~~~~~~~~~~~

    The tests for the exception classes.

    TODO:

    -   This is undertested.  HTML is never checked

    :copyright: 2007 Pallets
    :license: BSD-3-Clause
"""
import pytest

from werkzeug import exceptions
from werkzeug._compat import text_type
from werkzeug.datastructures import WWWAuthenticate
from werkzeug.wrappers import Response


def test_proxy_exception():
    orig_resp = Response("Hello World")
    with pytest.raises(exceptions.HTTPException) as excinfo:
        exceptions.abort(orig_resp)
    resp = excinfo.value.get_response({})
    assert resp is orig_resp
    assert resp.get_data() == b"Hello World"


@pytest.mark.parametrize(
    "test",
    [
        (exceptions.BadRequest, 400),
        (exceptions.Unauthorized, 401, 'Basic "test realm"'),
        (exceptions.Forbidden, 403),
        (exceptions.NotFound, 404),
        (exceptions.MethodNotAllowed, 405, ["GET", "HEAD"]),
        (exceptions.NotAcceptable, 406),
        (exceptions.RequestTimeout, 408),
        (exceptions.Gone, 410),
        (exceptions.LengthRequired, 411),
        (exceptions.PreconditionFailed, 412),
        (exceptions.RequestEntityTooLarge, 413),
        (exceptions.RequestURITooLarge, 414),
        (exceptions.UnsupportedMediaType, 415),
        (exceptions.UnprocessableEntity, 422),
        (exceptions.Locked, 423),
        (exceptions.InternalServerError, 500),
        (exceptions.NotImplemented, 501),
        (exceptions.BadGateway, 502),
        (exceptions.ServiceUnavailable, 503),
    ],
)
def test_aborter_general(test):
    exc_type = test[0]
    args = test[1:]

    with pytest.raises(exc_type) as exc_info:
        exceptions.abort(*args)
    assert type(exc_info.value) is exc_type


def test_aborter_custom():
    myabort = exceptions.Aborter({1: exceptions.NotFound})
    pytest.raises(LookupError, myabort, 404)
    pytest.raises(exceptions.NotFound, myabort, 1)

    myabort = exceptions.Aborter(extra={1: exceptions.NotFound})
    pytest.raises(exceptions.NotFound, myabort, 404)
    pytest.raises(exceptions.NotFound, myabort, 1)


def test_exception_repr():
    exc = exceptions.NotFound()
    assert text_type(exc) == (
        "404 Not Found: The requested URL was not found on the server."
        " If you entered the URL manually please check your spelling"
        " and try again."
    )
    assert repr(exc) == "<NotFound '404: Not Found'>"

    exc = exceptions.NotFound("Not There")
    assert text_type(exc) == "404 Not Found: Not There"
    assert repr(exc) == "<NotFound '404: Not Found'>"

    exc = exceptions.HTTPException("An error message")
    assert text_type(exc) == "??? Unknown Error: An error message"
    assert repr(exc) == "<HTTPException '???: Unknown Error'>"


def test_method_not_allowed_methods():
    exc = exceptions.MethodNotAllowed(["GET", "HEAD", "POST"])
    h = dict(exc.get_headers({}))
    assert h["Allow"] == "GET, HEAD, POST"
    assert "The method is not allowed" in exc.get_description()


def test_unauthorized_www_authenticate():
    basic = WWWAuthenticate()
    basic.set_basic("test")
    digest = WWWAuthenticate()
    digest.set_digest("test", "test")

    exc = exceptions.Unauthorized(www_authenticate=basic)
    h = dict(exc.get_headers({}))
    assert h["WWW-Authenticate"] == str(basic)

    exc = exceptions.Unauthorized(www_authenticate=[digest, basic])
    h = dict(exc.get_headers({}))
    assert h["WWW-Authenticate"] == ", ".join((str(digest), str(basic)))

    exc = exceptions.Unauthorized()
    h = dict(exc.get_headers({}))
    assert "WWW-Authenticate" not in h



# -*- coding: utf-8 -*-
"""
    tests.utils
    ~~~~~~~~~~~

    General utilities.

    :copyright: 2007 Pallets
    :license: BSD-3-Clause
"""
import inspect
from datetime import datetime

import pytest

from werkzeug import utils
from werkzeug._compat import text_type
from werkzeug.datastructures import Headers
from werkzeug.http import http_date
from werkzeug.http import parse_date
from werkzeug.test import Client
from werkzeug.wrappers import BaseResponse


def test_redirect():
    resp = utils.redirect(u"/füübär")
    assert b"/f%C3%BC%C3%BCb%C3%A4r" in resp.get_data()
    assert resp.headers["Location"] == "/f%C3%BC%C3%BCb%C3%A4r"
    assert resp.status_code == 302

    resp = utils.redirect(u"http://☃.net/", 307)
    assert b"http://xn--n3h.net/" in resp.get_data()
    assert resp.headers["Location"] == "http://xn--n3h.net/"
    assert resp.status_code == 307

    resp = utils.redirect("http://example.com/", 305)
    assert resp.headers["Location"] == "http://example.com/"
    assert resp.status_code == 305


def test_redirect_xss():
    location = 'http://example.com/?xss="><script>alert(1)</script>'
    resp = utils.redirect(location)
    assert b"<script>alert(1)</script>" not in resp.get_data()

    location = 'http://example.com/?xss="onmouseover="alert(1)'
    resp = utils.redirect(location)
    assert (
        b'href="http://example.com/?xss="onmouseover="alert(1)"' not in resp.get_data()
    )


def test_redirect_with_custom_response_class():
    class MyResponse(BaseResponse):
        pass

    location = "http://example.com/redirect"
    resp = utils.redirect(location, Response=MyResponse)

    assert isinstance(resp, MyResponse)
    assert resp.headers["Location"] == location


def test_cached_property():
    foo = []

    class A(object):
        def prop(self):
            foo.append(42)
            return 42

        prop = utils.cached_property(prop)

    a = A()
    p = a.prop
    q = a.prop
    assert p == q == 42
    assert foo == [42]

    foo = []

    class A(object):
        def _prop(self):
            foo.append(42)
            return 42

        prop = utils.cached_property(_prop, name="prop")
        del _prop

    a = A()
    p = a.prop
    q = a.prop
    assert p == q == 42
    assert foo == [42]


def test_can_set_cached_property():
    class A(object):
        @utils.cached_property
        def _prop(self):
            return "cached_property return value"

    a = A()
    a._prop = "value"
    assert a._prop == "value"


def test_inspect_treats_cached_property_as_property():
    class A(object):
        @utils.cached_property
        def _prop(self):
            return "cached_property return value"

    attrs = inspect.classify_class_attrs(A)
    for attr in attrs:
        if attr.name == "_prop":
            break
    assert attr.kind == "property"


def test_environ_property():
    class A(object):
        environ = {"string": "abc", "number": "42"}

        string = utils.environ_property("string")
        missing = utils.environ_property("missing", "spam")
        read_only = utils.environ_property("number")
        number = utils.environ_property("number", load_func=int)
        broken_number = utils.environ_property("broken_number", load_func=int)
        date = utils.environ_property(
            "date", None, parse_date, http_date, read_only=False
        )
        foo = utils.environ_property("foo")

    a = A()
    assert a.string == "abc"
    assert a.missing == "spam"

    def test_assign():
        a.read_only = "something"

    pytest.raises(AttributeError, test_assign)
    assert a.number == 42
    assert a.broken_number is None
    assert a.date is None
    a.date = datetime(2008, 1, 22, 10, 0, 0, 0)
    assert a.environ["date"] == "Tue, 22 Jan 2008 10:00:00 GMT"


def test_escape():
    class Foo(str):
        def __html__(self):
            return text_type(self)

    assert utils.escape(None) == ""
    assert utils.escape(42) == "42"
    assert utils.escape("<>") == "&lt;&gt;"
    assert utils.escape('"foo"') == "&quot;foo&quot;"
    assert utils.escape(Foo("<foo>")) == "<foo>"


def test_unescape():
    assert utils.unescape("&lt;&auml;&gt;") == u"<ä>"


def test_import_string():
    from datetime import date
    from werkzeug.debug import DebuggedApplication

    assert utils.import_string("datetime.date") is date
    assert utils.import_string(u"datetime.date") is date
    assert utils.import_string("datetime:date") is date
    assert utils.import_string("XXXXXXXXXXXX", True) is None
    assert utils.import_string("datetime.XXXXXXXXXXXX", True) is None
    assert (
        utils.import_string(u"werkzeug.debug.DebuggedApplication")
        is DebuggedApplication
    )
    pytest.raises(ImportError, utils.import_string, "XXXXXXXXXXXXXXXX")
    pytest.raises(ImportError, utils.import_string, "datetime.XXXXXXXXXX")


def test_import_string_provides_traceback(tmpdir, monkeypatch):
    monkeypatch.syspath_prepend(str(tmpdir))
    # Couple of packages
    dir_a = tmpdir.mkdir("a")
    dir_b = tmpdir.mkdir("b")
    # Totally packages, I promise
    dir_a.join("__init__.py").write("")
    dir_b.join("__init__.py").write("")
    # 'aa.a' that depends on 'bb.b', which in turn has a broken import
    dir_a.join("aa.py").write("from b import bb")
    dir_b.join("bb.py").write("from os import a_typo")

    # Do we get all the useful information in the traceback?
    with pytest.raises(ImportError) as baz_exc:
        utils.import_string("a.aa")
    traceback = "".join((str(line) for line in baz_exc.traceback))
    assert "bb.py':1" in traceback  # a bit different than typical python tb
    assert "from os import a_typo" in traceback


def test_import_string_attribute_error(tmpdir, monkeypatch):
    monkeypatch.syspath_prepend(str(tmpdir))
    tmpdir.join("foo_test.py").write("from bar_test import value")
    tmpdir.join("bar_test.py").write("raise AttributeError('bad')")

    with pytest.raises(AttributeError) as info:
        utils.import_string("foo_test")

    assert "bad" in str(info.value)

    with pytest.raises(AttributeError) as info:
        utils.import_string("bar_test")

    assert "bad" in str(info.value)


def test_find_modules():
    assert list(utils.find_modules("werkzeug.debug")) == [
        "werkzeug.debug.console",
        "werkzeug.debug.repr",
        "werkzeug.debug.tbtools",
    ]


def test_html_builder():
    html = utils.html
    xhtml = utils.xhtml
    assert html.p("Hello World") == "<p>Hello World</p>"
    assert html.a("Test", href="#") == '<a href="#">Test</a>'
    assert html.br() == "<br>"
    assert xhtml.br() == "<br />"
    assert html.img(src="foo") == '<img src="foo">'
    assert xhtml.img(src="foo") == '<img src="foo" />'
    assert html.html(
        html.head(html.title("foo"), html.script(type="text/javascript"))
    ) == (
        '<html><head><title>foo</title><script type="text/javascript">'
        "</script></head></html>"
    )
    assert html("<foo>") == "&lt;foo&gt;"
    assert html.input(disabled=True) == "<input disabled>"
    assert xhtml.input(disabled=True) == '<input disabled="disabled" />'
    assert html.input(disabled="") == "<input>"
    assert xhtml.input(disabled="") == "<input />"
    assert html.input(disabled=None) == "<input>"
    assert xhtml.input(disabled=None) == "<input />"
    assert (
        html.script('alert("Hello World");') == '<script>alert("Hello World");</script>'
    )
    assert (
        xhtml.script('alert("Hello World");')
        == '<script>/*<![CDATA[*/alert("Hello World");/*]]>*/</script>'
    )


def test_validate_arguments():
    def take_none():
        pass

    def take_two(a, b):
        pass

    def take_two_one_default(a, b=0):
        pass

    assert utils.validate_arguments(take_two, (1, 2), {}) == ((1, 2), {})
    assert utils.validate_arguments(take_two, (1,), {"b": 2}) == ((1, 2), {})
    assert utils.validate_arguments(take_two_one_default, (1,), {}) == ((1, 0), {})
    assert utils.validate_arguments(take_two_one_default, (1, 2), {}) == ((1, 2), {})

    pytest.raises(
        utils.ArgumentValidationError, utils.validate_arguments, take_two, (), {}
    )

    assert utils.validate_arguments(take_none, (1, 2), {"c": 3}) == ((), {})
    pytest.raises(
        utils.ArgumentValidationError,
        utils.validate_arguments,
        take_none,
        (1,),
        {},
        drop_extra=False,
    )
    pytest.raises(
        utils.ArgumentValidationError,
        utils.validate_arguments,
        take_none,
        (),
        {"a": 1},
        drop_extra=False,
    )


def test_header_set_duplication_bug():
    headers = Headers([("Content-Type", "text/html"), ("Foo", "bar"), ("Blub", "blah")])
    headers["blub"] = "hehe"
    headers["blafasel"] = "humm"
    assert headers == Headers(
        [
            ("Content-Type", "text/html"),
            ("Foo", "bar"),
            ("blub", "hehe"),
            ("blafasel", "humm"),
        ]
    )


def test_append_slash_redirect():
    def app(env, sr):
        return utils.append_slash_redirect(env)(env, sr)

    client = Client(app, BaseResponse)
    response = client.get("foo", base_url="http://example.org/app")
    assert response.status_code == 301
    assert response.headers["Location"] == "http://example.org/app/foo/"


def test_cached_property_doc():
    @utils.cached_property
    def foo():
        """testing"""
        return 42

    assert foo.__doc__ == "testing"
    assert foo.__name__ == "foo"
    assert foo.__module__ == __name__


def test_secure_filename():
    assert utils.secure_filename("My cool movie.mov") == "My_cool_movie.mov"
    assert utils.secure_filename("../../../etc/passwd") == "etc_passwd"
    assert (
        utils.secure_filename(u"i contain cool \xfcml\xe4uts.txt")
        == "i_contain_cool_umlauts.txt"
    )
    assert utils.secure_filename("__filename__") == "filename"
    assert utils.secure_filename("foo$&^*)bar") == "foobar"



# -*- coding: utf-8 -*-
"""
    tests.atom
    ~~~~~~~~~~

    Tests the cache system

    :copyright: 2007 Pallets
    :license: BSD-3-Clause
"""
import datetime

import pytest

from werkzeug.contrib.atom import AtomFeed
from werkzeug.contrib.atom import FeedEntry
from werkzeug.contrib.atom import format_iso8601


class TestAtomFeed(object):
    """
    Testcase for the `AtomFeed` class
    """

    def test_atom_no_args(self):
        with pytest.raises(ValueError):
            AtomFeed()

    def test_atom_title_no_id(self):
        with pytest.raises(ValueError):
            AtomFeed(title="test_title")

    def test_atom_add_one(self):
        a = AtomFeed(title="test_title", id=1)
        f = FeedEntry(title="test_title", id=1, updated=datetime.datetime.now())
        assert len(a.entries) == 0
        a.add(f)
        assert len(a.entries) == 1

    def test_atom_add_one_kwargs(self):
        a = AtomFeed(title="test_title", id=1)
        assert len(a.entries) == 0
        a.add(title="test_title", id=1, updated=datetime.datetime.now())
        assert len(a.entries) == 1
        assert isinstance(a.entries[0], FeedEntry)

    def test_atom_to_str(self):
        updated_time = datetime.datetime.now()
        expected_repr = """
        <?xml version="1.0" encoding="utf-8"?>
        <feed xmlns="http://www.w3.org/2005/Atom">
            <title type="text">test_title</title>
            <id>1</id>
            <updated>%s</updated>
            <generator>Werkzeug</generator>
        </feed>
        """ % format_iso8601(
            updated_time
        )
        a = AtomFeed(title="test_title", id=1, updated=updated_time)
        assert str(a).strip().replace(" ", "") == expected_repr.strip().replace(" ", "")


class TestFeedEntry(object):
    """
    Test case for the `FeedEntry` object
    """

    def test_feed_entry_no_args(self):
        with pytest.raises(ValueError):
            FeedEntry()

    def test_feed_entry_no_id(self):
        with pytest.raises(ValueError):
            FeedEntry(title="test_title")

    def test_feed_entry_no_updated(self):
        with pytest.raises(ValueError):
            FeedEntry(title="test_title", id=1)

    def test_feed_entry_to_str(self):
        updated_time = datetime.datetime.now()
        expected_feed_entry_str = """
        <entry>
            <title type="text">test_title</title>
            <id>1</id>
            <updated>%s</updated>
        </entry>
        """ % format_iso8601(
            updated_time
        )

        f = FeedEntry(title="test_title", id=1, updated=updated_time)
        assert str(f).strip().replace(
            " ", ""
        ) == expected_feed_entry_str.strip().replace(" ", "")


def test_format_iso8601():
    # naive datetime should be treated as utc
    dt = datetime.datetime(2014, 8, 31, 2, 5, 6)
    assert format_iso8601(dt) == "2014-08-31T02:05:06Z"

    # tz-aware datetime
    dt = datetime.datetime(2014, 8, 31, 11, 5, 6, tzinfo=KST())
    assert format_iso8601(dt) == "2014-08-31T11:05:06+09:00"


class KST(datetime.tzinfo):

    """KST implementation for test_format_iso8601()."""

    def utcoffset(self, dt):
        return datetime.timedelta(hours=9)

    def tzname(self, dt):
        return "KST"

    def dst(self, dt):
        return datetime.timedelta(0)



# -*- coding: utf-8 -*-
"""
    werkzeug
    ~~~~~~~~

    Werkzeug is the Swiss Army knife of Python web development.

    It provides useful classes and functions for any WSGI application to make
    the life of a python web developer much easier.  All of the provided
    classes are independent from each other so you can mix it with any other
    library.


    :copyright: 2007 Pallets
    :license: BSD-3-Clause
"""
import sys
from types import ModuleType

__version__ = "0.15.6"

# This import magic raises concerns quite often which is why the implementation
# and motivation is explained here in detail now.
#
# The majority of the functions and classes provided by Werkzeug work on the
# HTTP and WSGI layer.  There is no useful grouping for those which is why
# they are all importable from "werkzeug" instead of the modules where they are
# implemented.  The downside of that is, that now everything would be loaded at
# once, even if unused.
#
# The implementation of a lazy-loading module in this file replaces the
# werkzeug package when imported from within.  Attribute access to the werkzeug
# module will then lazily import from the modules that implement the objects.

# import mapping to objects in other modules
all_by_module = {
    "werkzeug.debug": ["DebuggedApplication"],
    "werkzeug.local": [
        "Local",
        "LocalManager",
        "LocalProxy",
        "LocalStack",
        "release_local",
    ],
    "werkzeug.serving": ["run_simple"],
    "werkzeug.test": ["Client", "EnvironBuilder", "create_environ", "run_wsgi_app"],
    "werkzeug.testapp": ["test_app"],
    "werkzeug.exceptions": ["abort", "Aborter"],
    "werkzeug.urls": [
        "url_decode",
        "url_encode",
        "url_quote",
        "url_quote_plus",
        "url_unquote",
        "url_unquote_plus",
        "url_fix",
        "Href",
        "iri_to_uri",
        "uri_to_iri",
    ],
    "werkzeug.formparser": ["parse_form_data"],
    "werkzeug.utils": [
        "escape",
        "environ_property",
        "append_slash_redirect",
        "redirect",
        "cached_property",
        "import_string",
        "unescape",
        "format_string",
        "find_modules",
        "header_property",
        "html",
        "xhtml",
        "HTMLBuilder",
        "validate_arguments",
        "ArgumentValidationError",
        "bind_arguments",
        "secure_filename",
    ],
    "werkzeug.wsgi": [
        "get_current_url",
        "get_host",
        "pop_path_info",
        "peek_path_info",
        "ClosingIterator",
        "FileWrapper",
        "make_line_iter",
        "LimitedStream",
        "responder",
        "wrap_file",
        "extract_path_info",
    ],
    "werkzeug.datastructures": [
        "MultiDict",
        "CombinedMultiDict",
        "Headers",
        "EnvironHeaders",
        "ImmutableList",
        "ImmutableDict",
        "ImmutableMultiDict",
        "TypeConversionDict",
        "ImmutableTypeConversionDict",
        "Accept",
        "MIMEAccept",
        "CharsetAccept",
        "LanguageAccept",
        "RequestCacheControl",
        "ResponseCacheControl",
        "ETags",
        "HeaderSet",
        "WWWAuthenticate",
        "Authorization",
        "FileMultiDict",
        "CallbackDict",
        "FileStorage",
        "OrderedMultiDict",
        "ImmutableOrderedMultiDict",
    ],
    "werkzeug.useragents": ["UserAgent"],
    "werkzeug.http": [
        "parse_etags",
        "parse_date",
        "http_date",
        "cookie_date",
        "parse_cache_control_header",
        "is_resource_modified",
        "parse_accept_header",
        "parse_set_header",
        "quote_etag",
        "unquote_etag",
        "generate_etag",
        "dump_header",
        "parse_list_header",
        "parse_dict_header",
        "parse_authorization_header",
        "parse_www_authenticate_header",
        "remove_entity_headers",
        "is_entity_header",
        "remove_hop_by_hop_headers",
        "parse_options_header",
        "dump_options_header",
        "is_hop_by_hop_header",
        "unquote_header_value",
        "quote_header_value",
        "HTTP_STATUS_CODES",
        "dump_cookie",
        "parse_cookie",
    ],
    "werkzeug.wrappers": [
        "BaseResponse",
        "BaseRequest",
        "Request",
        "Response",
        "AcceptMixin",
        "ETagRequestMixin",
        "ETagResponseMixin",
        "ResponseStreamMixin",
        "CommonResponseDescriptorsMixin",
        "UserAgentMixin",
        "AuthorizationMixin",
        "WWWAuthenticateMixin",
        "CommonRequestDescriptorsMixin",
    ],
    "werkzeug.middleware.dispatcher": ["DispatcherMiddleware"],
    "werkzeug.middleware.shared_data": ["SharedDataMiddleware"],
    "werkzeug.security": ["generate_password_hash", "check_password_hash"],
    # the undocumented easteregg ;-)
    "werkzeug._internal": ["_easteregg"],
}

# modules that should be imported when accessed as attributes of werkzeug
attribute_modules = frozenset(["exceptions", "routing"])

object_origins = {}
for module, items in all_by_module.items():
    for item in items:
        object_origins[item] = module


class module(ModuleType):
    """Automatically import objects from the modules."""

    def __getattr__(self, name):
        if name in object_origins:
            module = __import__(object_origins[name], None, None, [name])
            for extra_name in all_by_module[module.__name__]:
                setattr(self, extra_name, getattr(module, extra_name))
            return getattr(module, name)
        elif name in attribute_modules:
            __import__("werkzeug." + name)
        return ModuleType.__getattribute__(self, name)

    def __dir__(self):
        """Just show what we want to show."""
        result = list(new_module.__all__)
        result.extend(
            (
                "__file__",
                "__doc__",
                "__all__",
                "__docformat__",
                "__name__",
                "__path__",
                "__package__",
                "__version__",
            )
        )
        return result


# keep a reference to this module so that it's not garbage collected
old_module = sys.modules["werkzeug"]


# setup the new module and patch it into the dict of loaded modules
new_module = sys.modules["werkzeug"] = module("werkzeug")
new_module.__dict__.update(
    {
        "__file__": __file__,
        "__package__": "werkzeug",
        "__path__": __path__,
        "__doc__": __doc__,
        "__version__": __version__,
        "__all__": tuple(object_origins) + tuple(attribute_modules),
        "__docformat__": "restructuredtext en",
    }
)


# Due to bootstrapping issues we need to import exceptions here.
# Don't ask :-(
__import__("werkzeug.exceptions")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    Cookie Based Auth
    ~~~~~~~~~~~~~~~~~

    This is a very simple application that uses a secure cookie to do the
    user authentification.

    :copyright: 2007 Pallets
    :license: BSD-3-Clause
"""
from werkzeug.contrib.securecookie import SecureCookie
from werkzeug.serving import run_simple
from werkzeug.utils import cached_property
from werkzeug.utils import escape
from werkzeug.utils import redirect
from werkzeug.wrappers import Request
from werkzeug.wrappers import Response


# don't use this key but a different one; you could just use
# os.unrandom(20) to get something random.  Changing this key
# invalidates all sessions at once.
SECRET_KEY = "\xfa\xdd\xb8z\xae\xe0}4\x8b\xea"

# the cookie name for the session
COOKIE_NAME = "session"

# the users that may access
USERS = {"admin": "default", "user1": "default"}


class AppRequest(Request):
    """A request with a secure cookie session."""

    def logout(self):
        """Log the user out."""
        self.session.pop("username", None)

    def login(self, username):
        """Log the user in."""
        self.session["username"] = username

    @property
    def logged_in(self):
        """Is the user logged in?"""
        return self.user is not None

    @property
    def user(self):
        """The user that is logged in."""
        return self.session.get("username")

    @cached_property
    def session(self):
        data = self.cookies.get(COOKIE_NAME)
        if not data:
            return SecureCookie(secret_key=SECRET_KEY)
        return SecureCookie.unserialize(data, SECRET_KEY)


def login_form(request):
    error = ""
    if request.method == "POST":
        username = request.form.get("username")
        password = request.form.get("password")
        if password and USERS.get(username) == password:
            request.login(username)
            return redirect("")
        error = "<p>Invalid credentials"
    return Response(
        """<title>Login</title><h1>Login</h1>
        <p>Not logged in.
        %s
        <form action="" method="post">
          <p>
            <input type="hidden" name="do" action="login">
            <input type="text" name="username" size=20>
            <input type="password" name="password", size=20>
            <input type="submit" value="Login">
        </form>"""
        % error,
        mimetype="text/html",
    )


def index(request):
    return Response(
        """<title>Logged in</title>
        <h1>Logged in</h1>
        <p>Logged in as %s
        <p><a href="/?do=logout">Logout</a>"""
        % escape(request.user),
        mimetype="text/html",
    )


@AppRequest.application
def application(request):
    if request.args.get("do") == "logout":
        request.logout()
        response = redirect(".")
    elif request.logged_in:
        response = index(request)
    else:
        response = login_form(request)
    request.session.save_cookie(response)
    return response


if __name__ == "__main__":
    run_simple("localhost", 4000, application)



# -*- coding: utf-8 -*-
"""
    tests.cache
    ~~~~~~~~~~~

    Tests the cache system

    :copyright: 2007 Pallets
    :license: BSD-3-Clause
"""
import errno

import pytest

from werkzeug._compat import text_type
from werkzeug.contrib import cache

try:
    import redis
except ImportError:
    redis = None

try:
    import pylibmc as memcache
except ImportError:
    try:
        from google.appengine.api import memcache
    except ImportError:
        try:
            import memcache
        except ImportError:
            memcache = None

pytestmark = pytest.mark.skip("werkzeug.contrib.cache moved to cachelib")


class CacheTestsBase(object):
    _can_use_fast_sleep = True
    _guaranteed_deletes = True

    @pytest.fixture
    def fast_sleep(self, monkeypatch):
        if self._can_use_fast_sleep:

            def sleep(delta):
                orig_time = cache.time
                monkeypatch.setattr(cache, "time", lambda: orig_time() + delta)

            return sleep
        else:
            import time

            return time.sleep

    @pytest.fixture
    def make_cache(self):
        """Return a cache class or factory."""
        raise NotImplementedError()

    @pytest.fixture
    def c(self, make_cache):
        """Return a cache instance."""
        return make_cache()


class GenericCacheTests(CacheTestsBase):
    def test_generic_get_dict(self, c):
        assert c.set("a", "a")
        assert c.set("b", "b")
        d = c.get_dict("a", "b")
        assert "a" in d
        assert "a" == d["a"]
        assert "b" in d
        assert "b" == d["b"]

    def test_generic_set_get(self, c):
        for i in range(3):
            assert c.set(str(i), i * i)

        for i in range(3):
            result = c.get(str(i))
            assert result == i * i, result

    def test_generic_get_set(self, c):
        assert c.set("foo", ["bar"])
        assert c.get("foo") == ["bar"]

    def test_generic_get_many(self, c):
        assert c.set("foo", ["bar"])
        assert c.set("spam", "eggs")
        assert c.get_many("foo", "spam") == [["bar"], "eggs"]

    def test_generic_set_many(self, c):
        assert c.set_many({"foo": "bar", "spam": ["eggs"]})
        assert c.get("foo") == "bar"
        assert c.get("spam") == ["eggs"]

    def test_generic_add(self, c):
        # sanity check that add() works like set()
        assert c.add("foo", "bar")
        assert c.get("foo") == "bar"
        assert not c.add("foo", "qux")
        assert c.get("foo") == "bar"

    def test_generic_delete(self, c):
        assert c.add("foo", "bar")
        assert c.get("foo") == "bar"
        assert c.delete("foo")
        assert c.get("foo") is None

    def test_generic_delete_many(self, c):
        assert c.add("foo", "bar")
        assert c.add("spam", "eggs")
        assert c.delete_many("foo", "spam")
        assert c.get("foo") is None
        assert c.get("spam") is None

    def test_generic_inc_dec(self, c):
        assert c.set("foo", 1)
        assert c.inc("foo") == c.get("foo") == 2
        assert c.dec("foo") == c.get("foo") == 1
        assert c.delete("foo")

    def test_generic_true_false(self, c):
        assert c.set("foo", True)
        assert c.get("foo") in (True, 1)
        assert c.set("bar", False)
        assert c.get("bar") in (False, 0)

    def test_generic_timeout(self, c, fast_sleep):
        c.set("foo", "bar", 0)
        assert c.get("foo") == "bar"
        c.set("baz", "qux", 1)
        assert c.get("baz") == "qux"
        fast_sleep(3)
        # timeout of zero means no timeout
        assert c.get("foo") == "bar"
        if self._guaranteed_deletes:
            assert c.get("baz") is None

    def test_generic_has(self, c):
        assert c.has("foo") in (False, 0)
        assert c.has("spam") in (False, 0)
        assert c.set("foo", "bar")
        assert c.has("foo") in (True, 1)
        assert c.has("spam") in (False, 0)
        c.delete("foo")
        assert c.has("foo") in (False, 0)
        assert c.has("spam") in (False, 0)


class TestSimpleCache(GenericCacheTests):
    @pytest.fixture
    def make_cache(self):
        return cache.SimpleCache

    def test_purge(self):
        c = cache.SimpleCache(threshold=2)
        c.set("a", "a")
        c.set("b", "b")
        c.set("c", "c")
        c.set("d", "d")
        # Cache purges old items *before* it sets new ones.
        assert len(c._cache) == 3


class TestFileSystemCache(GenericCacheTests):
    @pytest.fixture
    def make_cache(self, tmpdir):
        return lambda **kw: cache.FileSystemCache(cache_dir=str(tmpdir), **kw)

    def test_filesystemcache_prune(self, make_cache):
        THRESHOLD = 13
        c = make_cache(threshold=THRESHOLD)

        for i in range(2 * THRESHOLD):
            assert c.set(str(i), i)

        nof_cache_files = c.get(c._fs_count_file)
        assert nof_cache_files <= THRESHOLD

    def test_filesystemcache_clear(self, c):
        assert c.set("foo", "bar")
        nof_cache_files = c.get(c._fs_count_file)
        assert nof_cache_files == 1
        assert c.clear()
        nof_cache_files = c.get(c._fs_count_file)
        assert nof_cache_files == 0
        cache_files = c._list_dir()
        assert len(cache_files) == 0

    def test_no_threshold(self, make_cache):
        THRESHOLD = 0
        c = make_cache(threshold=THRESHOLD)

        for i in range(10):
            assert c.set(str(i), i)

        cache_files = c._list_dir()
        assert len(cache_files) == 10

        # File count is not maintained with threshold = 0
        nof_cache_files = c.get(c._fs_count_file)
        assert nof_cache_files is None

    def test_count_file_accuracy(self, c):
        assert c.set("foo", "bar")
        assert c.set("moo", "car")
        c.add("moo", "tar")
        assert c.get(c._fs_count_file) == 2
        assert c.add("too", "far")
        assert c.get(c._fs_count_file) == 3
        assert c.delete("moo")
        assert c.get(c._fs_count_file) == 2
        assert c.clear()
        assert c.get(c._fs_count_file) == 0


# don't use pytest.mark.skipif on subclasses
# https://bitbucket.org/hpk42/pytest/issue/568
# skip happens in requirements fixture instead
class TestRedisCache(GenericCacheTests):
    _can_use_fast_sleep = False

    @pytest.fixture(scope="class", autouse=True)
    def requirements(self, xprocess):
        if redis is None:
            pytest.skip('Python package "redis" is not installed.')

        def prepare(cwd):
            return "[Rr]eady to accept connections", ["redis-server"]

        try:
            xprocess.ensure("redis_server", prepare)
        except IOError as e:
            # xprocess raises FileNotFoundError
            if e.errno == errno.ENOENT:
                pytest.skip("Redis is not installed.")
            else:
                raise

        yield
        xprocess.getinfo("redis_server").terminate()

    @pytest.fixture(params=(None, False, True))
    def make_cache(self, request):
        if request.param is None:
            host = "localhost"
        elif request.param:
            host = redis.StrictRedis()
        else:
            host = redis.Redis()

        c = cache.RedisCache(host=host, key_prefix="werkzeug-test-case:")
        yield lambda: c
        c.clear()

    def test_compat(self, c):
        assert c._client.set(c.key_prefix + "foo", "Awesome")
        assert c.get("foo") == b"Awesome"
        assert c._client.set(c.key_prefix + "foo", "42")
        assert c.get("foo") == 42

    def test_empty_host(self):
        with pytest.raises(ValueError) as exc_info:
            cache.RedisCache(host=None)
        assert text_type(exc_info.value) == "RedisCache host parameter may not be None"


class TestMemcachedCache(GenericCacheTests):
    _can_use_fast_sleep = False
    _guaranteed_deletes = False

    @pytest.fixture(scope="class", autouse=True)
    def requirements(self, xprocess):
        if memcache is None:
            pytest.skip(
                "Python package for memcache is not installed. Need one of "
                '"pylibmc", "google.appengine", or "memcache".'
            )

        def prepare(cwd):
            return "", ["memcached"]

        try:
            xprocess.ensure("memcached", prepare)
        except IOError as e:
            # xprocess raises FileNotFoundError
            if e.errno == errno.ENOENT:
                pytest.skip("Memcached is not installed.")
            else:
                raise

        yield
        xprocess.getinfo("memcached").terminate()

    @pytest.fixture
    def make_cache(self):
        c = cache.MemcachedCache(key_prefix="werkzeug-test-case:")
        yield lambda: c
        c.clear()

    def test_compat(self, c):
        assert c._client.set(c.key_prefix + "foo", "bar")
        assert c.get("foo") == "bar"

    def test_huge_timeouts(self, c):
        # Timeouts greater than epoch are interpreted as POSIX timestamps
        # (i.e. not relative to now, but relative to epoch)
        epoch = 2592000
        c.set("foo", "bar", epoch + 100)
        assert c.get("foo") == "bar"


class TestUWSGICache(GenericCacheTests):
    _can_use_fast_sleep = False
    _guaranteed_deletes = False

    @pytest.fixture(scope="class", autouse=True)
    def requirements(self):
        try:
            import uwsgi  # NOQA
        except ImportError:
            pytest.skip(
                'Python "uwsgi" package is only avaialable when running '
                "inside uWSGI."
            )

    @pytest.fixture
    def make_cache(self):
        c = cache.UWSGICache(cache="werkzeugtest")
        yield lambda: c
        c.clear()


class TestNullCache(CacheTestsBase):
    @pytest.fixture(scope="class", autouse=True)
    def make_cache(self):
        return cache.NullCache

    def test_has(self, c):
        assert not c.has("foo")



# -*- coding: utf-8 -*-
"""
    Secure Cookie Example
    ~~~~~~~~~~~~~~~~~~~~~

    Stores session on the client.

    :copyright: 2007 Pallets
    :license: BSD-3-Clause
"""
from time import asctime

from werkzeug.contrib.securecookie import SecureCookie
from werkzeug.serving import run_simple
from werkzeug.wrappers import BaseRequest
from werkzeug.wrappers import BaseResponse

SECRET_KEY = "V\x8a$m\xda\xe9\xc3\x0f|f\x88\xbccj>\x8bI^3+"


class Request(BaseRequest):
    def __init__(self, environ):
        BaseRequest.__init__(self, environ)
        self.session = SecureCookie.load_cookie(self, secret_key=SECRET_KEY)


def index(request):
    return '<a href="set">Set the Time</a> or <a href="get">Get the time</a>'


def get_time(request):
    return "Time: %s" % request.session.get("time", "not set")


def set_time(request):
    request.session["time"] = time = asctime()
    return "Time set to %s" % time


def application(environ, start_response):
    request = Request(environ)
    response = BaseResponse(
        {"get": get_time, "set": set_time}.get(request.path.strip("/"), index)(request),
        mimetype="text/html",
    )
    request.session.save_cookie(response)
    return response(environ, start_response)


if __name__ == "__main__":
    run_simple("localhost", 5000, application)



# -*- coding: utf-8 -*-
"""
    tests.securecookie
    ~~~~~~~~~~~~~~~~~~

    Tests the secure cookie.

    :copyright: 2007 Pallets
    :license: BSD-3-Clause
"""
import json

import pytest

from werkzeug._compat import to_native
from werkzeug.contrib.securecookie import SecureCookie
from werkzeug.utils import parse_cookie
from werkzeug.wrappers import Request
from werkzeug.wrappers import Response


def test_basic_support():
    c = SecureCookie(secret_key=b"foo")
    assert c.new
    assert not c.modified
    assert not c.should_save
    c["x"] = 42
    assert c.modified
    assert c.should_save
    s = c.serialize()

    c2 = SecureCookie.unserialize(s, b"foo")
    assert c is not c2
    assert not c2.new
    assert not c2.modified
    assert not c2.should_save
    assert c2 == c

    c3 = SecureCookie.unserialize(s, b"wrong foo")
    assert not c3.modified
    assert not c3.new
    assert c3 == {}

    c4 = SecureCookie({"x": 42}, "foo")
    c4_serialized = c4.serialize()
    assert SecureCookie.unserialize(c4_serialized, "foo") == c4


def test_wrapper_support():
    req = Request.from_values()
    resp = Response()
    c = SecureCookie.load_cookie(req, secret_key=b"foo")
    assert c.new
    c["foo"] = 42
    assert c.secret_key == b"foo"
    c.save_cookie(resp)

    req = Request.from_values(
        headers={
            "Cookie": 'session="%s"'
            % parse_cookie(resp.headers["set-cookie"])["session"]
        }
    )
    c2 = SecureCookie.load_cookie(req, secret_key=b"foo")
    assert not c2.new
    assert c2 == c


def test_pickle_deprecated():
    with pytest.warns(UserWarning):
        SecureCookie({"foo": "bar"}, "secret").serialize()


def test_json():
    class JSONCompat(object):
        dumps = staticmethod(json.dumps)

        @staticmethod
        def loads(s):
            # json on Python < 3.6 fails on bytes
            return json.loads(to_native(s, "utf8"))

    class JSONSecureCookie(SecureCookie):
        serialization_method = JSONCompat

    secure = JSONSecureCookie({"foo": "bar"}, "secret").serialize()
    data = JSONSecureCookie.unserialize(secure, "secret")
    assert data == {"foo": "bar"}



# -*- coding: utf-8 -*-
"""
    tests.contrib.wrappers
    ~~~~~~~~~~~~~~~~~~~~~~

    Added tests for the sessions.

    :copyright: 2007 Pallets
    :license: BSD-3-Clause
"""
from werkzeug import routing
from werkzeug.contrib import wrappers
from werkzeug.wrappers import Request
from werkzeug.wrappers import Response


def test_reverse_slash_behavior():
    class MyRequest(wrappers.ReverseSlashBehaviorRequestMixin, Request):
        pass

    req = MyRequest.from_values("/foo/bar", "http://example.com/test")
    assert req.url == "http://example.com/test/foo/bar"
    assert req.path == "foo/bar"
    assert req.script_root == "/test/"

    # make sure the routing system works with the slashes in
    # reverse order as well.
    map = routing.Map([routing.Rule("/foo/bar", endpoint="foo")])
    adapter = map.bind_to_environ(req.environ)
    assert adapter.match() == ("foo", {})
    adapter = map.bind(req.host, req.script_root)
    assert adapter.match(req.path) == ("foo", {})


def test_dynamic_charset_request_mixin():
    class MyRequest(wrappers.DynamicCharsetRequestMixin, Request):
        pass

    env = {"CONTENT_TYPE": "text/html"}
    req = MyRequest(env)
    assert req.charset == "latin1"

    env = {"CONTENT_TYPE": "text/html; charset=utf-8"}
    req = MyRequest(env)
    assert req.charset == "utf-8"

    env = {"CONTENT_TYPE": "application/octet-stream"}
    req = MyRequest(env)
    assert req.charset == "latin1"
    assert req.url_charset == "latin1"

    MyRequest.url_charset = "utf-8"
    env = {"CONTENT_TYPE": "application/octet-stream"}
    req = MyRequest(env)
    assert req.charset == "latin1"
    assert req.url_charset == "utf-8"

    def return_ascii(x):
        return "ascii"

    env = {"CONTENT_TYPE": "text/plain; charset=x-weird-charset"}
    req = MyRequest(env)
    req.unknown_charset = return_ascii
    assert req.charset == "ascii"
    assert req.url_charset == "utf-8"


def test_dynamic_charset_response_mixin():
    class MyResponse(wrappers.DynamicCharsetResponseMixin, Response):
        default_charset = "utf-7"

    resp = MyResponse(mimetype="text/html")
    assert resp.charset == "utf-7"
    resp.charset = "utf-8"
    assert resp.charset == "utf-8"
    assert resp.mimetype == "text/html"
    assert resp.mimetype_params == {"charset": "utf-8"}
    resp.mimetype_params["charset"] = "iso-8859-15"
    assert resp.charset == "iso-8859-15"
    resp.set_data(u"Hällo Wörld")
    assert b"".join(resp.iter_encoded()) == u"Hällo Wörld".encode("iso-8859-15")
    del resp.headers["content-type"]
    try:
        resp.charset = "utf-8"
    except TypeError:
        pass
    else:
        raise AssertionError("expected type error on charset setting without ct")



import os

import pytest

# build the path to the uwsgi marker file
# when running in tox, this will be relative to the tox env
filename = os.path.join(os.environ.get("TOX_ENVTMPDIR", ""), "test_uwsgi_failed")


@pytest.hookimpl(tryfirst=True, hookwrapper=True)
def pytest_runtest_makereport(item, call):
    """``uwsgi --pyrun`` doesn't pass on the exit code when ``pytest`` fails,
    so Tox thinks the tests passed. For UWSGI tests, create a file to mark what
    tests fail. The uwsgi Tox env has a command to read this file and exit
    appropriately.
    """
    outcome = yield
    report = outcome.get_result()

    if item.cls.__name__ != "TestUWSGICache":
        return

    if report.failed:
        with open(filename, "a") as f:
            f.write(item.name + "\n")



import hypothesis
from hypothesis.strategies import dictionaries
from hypothesis.strategies import integers
from hypothesis.strategies import lists
from hypothesis.strategies import text

from werkzeug import urls
from werkzeug.datastructures import OrderedMultiDict


@hypothesis.given(text())
def test_quote_unquote_text(t):
    assert t == urls.url_unquote(urls.url_quote(t))


@hypothesis.given(dictionaries(text(), text()))
def test_url_encoding_dict_str_str(d):
    assert OrderedMultiDict(d) == urls.url_decode(urls.url_encode(d))


@hypothesis.given(dictionaries(text(), lists(elements=text())))
def test_url_encoding_dict_str_list(d):
    assert OrderedMultiDict(d) == urls.url_decode(urls.url_encode(d))


@hypothesis.given(dictionaries(text(), integers()))
def test_url_encoding_dict_str_int(d):
    assert OrderedMultiDict({k: str(v) for k, v in d.items()}) == urls.url_decode(
        urls.url_encode(d)
    )


@hypothesis.given(text(), text())
def test_multidict_encode_decode_text(t1, t2):
    d = OrderedMultiDict()
    d.add(t1, t2)
    assert d == urls.url_decode(urls.url_encode(d))



# -*- coding: utf-8 -*-
# flake8: noqa
"""
    tests.compat
    ~~~~~~~~~~~~

    Ensure that old stuff does not break on update.

    :copyright: 2007 Pallets
    :license: BSD-3-Clause
"""
from werkzeug.test import create_environ
from werkzeug.wrappers import Response


def test_old_imports():
    from werkzeug.utils import (
        Headers,
        MultiDict,
        CombinedMultiDict,
        Headers,
        EnvironHeaders,
    )
    from werkzeug.http import (
        Accept,
        MIMEAccept,
        CharsetAccept,
        LanguageAccept,
        ETags,
        HeaderSet,
        WWWAuthenticate,
        Authorization,
    )


def test_exposed_werkzeug_mod():
    import werkzeug

    for key in werkzeug.__all__:
        getattr(werkzeug, key)



from werkzeug.contrib import fixers
from werkzeug.datastructures import ResponseCacheControl
from werkzeug.http import parse_cache_control_header
from werkzeug.test import Client
from werkzeug.test import create_environ
from werkzeug.wrappers import Request
from werkzeug.wrappers import Response


@Request.application
def path_check_app(request):
    return Response(
        "PATH_INFO: %s\nSCRIPT_NAME: %s"
        % (request.environ.get("PATH_INFO", ""), request.environ.get("SCRIPT_NAME", ""))
    )


class TestServerFixer(object):
    def test_cgi_root_fix(self):
        app = fixers.CGIRootFix(path_check_app)
        response = Response.from_app(
            app, dict(create_environ(), SCRIPT_NAME="/foo", PATH_INFO="/bar")
        )
        assert response.get_data() == b"PATH_INFO: /bar\nSCRIPT_NAME: "

    def test_cgi_root_fix_custom_app_root(self):
        app = fixers.CGIRootFix(path_check_app, app_root="/baz/")
        response = Response.from_app(
            app, dict(create_environ(), SCRIPT_NAME="/foo", PATH_INFO="/bar")
        )
        assert response.get_data() == b"PATH_INFO: /bar\nSCRIPT_NAME: baz"

    def test_path_info_from_request_uri_fix(self):
        app = fixers.PathInfoFromRequestUriFix(path_check_app)
        for key in "REQUEST_URI", "REQUEST_URL", "UNENCODED_URL":
            env = dict(create_environ(), SCRIPT_NAME="/test", PATH_INFO="/?????")
            env[key] = "/test/foo%25bar?drop=this"
            response = Response.from_app(app, env)
            assert response.get_data() == b"PATH_INFO: /foo%bar\nSCRIPT_NAME: /test"

    def test_header_rewriter_fix(self):
        @Request.application
        def application(request):
            return Response("", headers=[("X-Foo", "bar")])

        application = fixers.HeaderRewriterFix(
            application, ("X-Foo",), (("X-Bar", "42"),)
        )
        response = Response.from_app(application, create_environ())
        assert response.headers["Content-Type"] == "text/plain; charset=utf-8"
        assert "X-Foo" not in response.headers
        assert response.headers["X-Bar"] == "42"


class TestBrowserFixer(object):
    def test_ie_fixes(self):
        @fixers.InternetExplorerFix
        @Request.application
        def application(request):
            response = Response("binary data here", mimetype="application/vnd.ms-excel")
            response.headers["Vary"] = "Cookie"
            response.headers["Content-Disposition"] = "attachment; filename=foo.xls"
            return response

        c = Client(application, Response)
        response = c.get(
            "/",
            headers=[
                ("User-Agent", "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)")
            ],
        )

        # IE gets no vary
        assert response.get_data() == b"binary data here"
        assert "vary" not in response.headers
        assert response.headers["content-disposition"] == "attachment; filename=foo.xls"
        assert response.headers["content-type"] == "application/vnd.ms-excel"

        # other browsers do
        c = Client(application, Response)
        response = c.get("/")
        assert response.get_data() == b"binary data here"
        assert "vary" in response.headers

        cc = ResponseCacheControl()
        cc.no_cache = True

        @fixers.InternetExplorerFix
        @Request.application
        def application(request):
            response = Response("binary data here", mimetype="application/vnd.ms-excel")
            response.headers["Pragma"] = ", ".join(pragma)
            response.headers["Cache-Control"] = cc.to_header()
            response.headers["Content-Disposition"] = "attachment; filename=foo.xls"
            return response

        # IE has no pragma or cache control
        pragma = ("no-cache",)
        c = Client(application, Response)
        response = c.get(
            "/",
            headers=[
                ("User-Agent", "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)")
            ],
        )
        assert response.get_data() == b"binary data here"
        assert "pragma" not in response.headers
        assert "cache-control" not in response.headers
        assert response.headers["content-disposition"] == "attachment; filename=foo.xls"

        # IE has simplified pragma
        pragma = ("no-cache", "x-foo")
        cc.proxy_revalidate = True
        response = c.get(
            "/",
            headers=[
                ("User-Agent", "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)")
            ],
        )
        assert response.get_data() == b"binary data here"
        assert response.headers["pragma"] == "x-foo"
        assert response.headers["cache-control"] == "proxy-revalidate"
        assert response.headers["content-disposition"] == "attachment; filename=foo.xls"

        # regular browsers get everything
        response = c.get("/")
        assert response.get_data() == b"binary data here"
        assert response.headers["pragma"] == "no-cache, x-foo"
        cc = parse_cache_control_header(
            response.headers["cache-control"], cls=ResponseCacheControl
        )
        assert cc.no_cache
        assert cc.proxy_revalidate
        assert response.headers["content-disposition"] == "attachment; filename=foo.xls"



# -*- coding: utf-8 -*-
"""
    tests.iterio
    ~~~~~~~~~~~~

    Tests the iterio object.

    :copyright: 2007 Pallets
    :license: BSD-3-Clause
"""
import pytest

from .. import strict_eq
from werkzeug.contrib.iterio import greenlet
from werkzeug.contrib.iterio import IterIO


class TestIterO(object):
    def test_basic_native(self):
        io = IterIO(["Hello", "World", "1", "2", "3"])
        io.seek(0)
        assert io.tell() == 0
        assert io.read(2) == "He"
        assert io.tell() == 2
        assert io.read(3) == "llo"
        assert io.tell() == 5
        io.seek(0)
        assert io.read(5) == "Hello"
        assert io.tell() == 5
        assert io._buf == "Hello"
        assert io.read() == "World123"
        assert io.tell() == 13
        io.close()
        assert io.closed

        io = IterIO(["Hello\n", "World!"])
        assert io.readline() == "Hello\n"
        assert io._buf == "Hello\n"
        assert io.read() == "World!"
        assert io._buf == "Hello\nWorld!"
        assert io.tell() == 12
        io.seek(0)
        assert io.readlines() == ["Hello\n", "World!"]

        io = IterIO(["Line one\nLine ", "two\nLine three"])
        assert list(io) == ["Line one\n", "Line two\n", "Line three"]
        io = IterIO(iter("Line one\nLine two\nLine three"))
        assert list(io) == ["Line one\n", "Line two\n", "Line three"]
        io = IterIO(["Line one\nL", "ine", " two", "\nLine three"])
        assert list(io) == ["Line one\n", "Line two\n", "Line three"]

        io = IterIO(["foo\n", "bar"])
        io.seek(-4, 2)
        assert io.read(4) == "\nbar"

        pytest.raises(IOError, io.seek, 2, 100)
        io.close()
        pytest.raises(ValueError, io.read)

    def test_basic_bytes(self):
        io = IterIO([b"Hello", b"World", b"1", b"2", b"3"])
        assert io.tell() == 0
        assert io.read(2) == b"He"
        assert io.tell() == 2
        assert io.read(3) == b"llo"
        assert io.tell() == 5
        io.seek(0)
        assert io.read(5) == b"Hello"
        assert io.tell() == 5
        assert io._buf == b"Hello"
        assert io.read() == b"World123"
        assert io.tell() == 13
        io.close()
        assert io.closed

        io = IterIO([b"Hello\n", b"World!"])
        assert io.readline() == b"Hello\n"
        assert io._buf == b"Hello\n"
        assert io.read() == b"World!"
        assert io._buf == b"Hello\nWorld!"
        assert io.tell() == 12
        io.seek(0)
        assert io.readlines() == [b"Hello\n", b"World!"]

        io = IterIO([b"foo\n", b"bar"])
        io.seek(-4, 2)
        assert io.read(4) == b"\nbar"

        pytest.raises(IOError, io.seek, 2, 100)
        io.close()
        pytest.raises(ValueError, io.read)

    def test_basic_unicode(self):
        io = IterIO([u"Hello", u"World", u"1", u"2", u"3"])
        assert io.tell() == 0
        assert io.read(2) == u"He"
        assert io.tell() == 2
        assert io.read(3) == u"llo"
        assert io.tell() == 5
        io.seek(0)
        assert io.read(5) == u"Hello"
        assert io.tell() == 5
        assert io._buf == u"Hello"
        assert io.read() == u"World123"
        assert io.tell() == 13
        io.close()
        assert io.closed

        io = IterIO([u"Hello\n", u"World!"])
        assert io.readline() == u"Hello\n"
        assert io._buf == u"Hello\n"
        assert io.read() == u"World!"
        assert io._buf == u"Hello\nWorld!"
        assert io.tell() == 12
        io.seek(0)
        assert io.readlines() == [u"Hello\n", u"World!"]

        io = IterIO([u"foo\n", u"bar"])
        io.seek(-4, 2)
        assert io.read(4) == u"\nbar"

        pytest.raises(IOError, io.seek, 2, 100)
        io.close()
        pytest.raises(ValueError, io.read)

    def test_sentinel_cases(self):
        io = IterIO([])
        strict_eq(io.read(), "")
        io = IterIO([], b"")
        strict_eq(io.read(), b"")
        io = IterIO([], u"")
        strict_eq(io.read(), u"")

        io = IterIO([])
        strict_eq(io.read(), "")
        io = IterIO([b""])
        strict_eq(io.read(), b"")
        io = IterIO([u""])
        strict_eq(io.read(), u"")

        io = IterIO([])
        strict_eq(io.readline(), "")
        io = IterIO([], b"")
        strict_eq(io.readline(), b"")
        io = IterIO([], u"")
        strict_eq(io.readline(), u"")

        io = IterIO([])
        strict_eq(io.readline(), "")
        io = IterIO([b""])
        strict_eq(io.readline(), b"")
        io = IterIO([u""])
        strict_eq(io.readline(), u"")


@pytest.mark.skipif(greenlet is None, reason="Greenlet is not installed.")
class TestIterI(object):
    def test_basic(self):
        def producer(out):
            out.write("1\n")
            out.write("2\n")
            out.flush()
            out.write("3\n")

        iterable = IterIO(producer)
        assert next(iterable) == "1\n2\n"
        assert next(iterable) == "3\n"
        pytest.raises(StopIteration, next, iterable)

    def test_sentinel_cases(self):
        def producer_dummy_flush(out):
            out.flush()

        iterable = IterIO(producer_dummy_flush)
        strict_eq(next(iterable), "")

        def producer_empty(out):
            pass

        iterable = IterIO(producer_empty)
        pytest.raises(StopIteration, next, iterable)

        iterable = IterIO(producer_dummy_flush, b"")
        strict_eq(next(iterable), b"")
        iterable = IterIO(producer_dummy_flush, u"")
        strict_eq(next(iterable), u"")
